@@ -41,16 +41,18 @@ class NrnParameter(bluepyopt.parameters.Parameter):
             name,
             value=None,
             frozen=False,
-            bounds=None):
+            bounds=None,
+            param_dependencies=None):
         """Contructor"""
 
         super(NrnParameter, self).__init__(
             name,
             value=value,
             frozen=frozen,
-            bounds=bounds)
+            bounds=bounds,
+            param_dependencies=param_dependencies)
 
-    def instantiate(self, sim=None, icell=None):
+    def instantiate(self, sim=None, icell=None, params=None):
         """Instantiate the parameter in the simulator"""
         pass
 
@@ -70,14 +72,16 @@ class MetaParameter(NrnParameter):
             attr_name=None,
             value=None,
             frozen=False,
-            bounds=None):
+            bounds=None,
+            param_dependencies=None):
         """Constructor"""
 
         super(MetaParameter, self).__init__(
             name,
             value=value,
             frozen=frozen,
-            bounds=bounds)
+            bounds=bounds,
+            param_dependencies=param_dependencies)
 
         self.obj = obj
         self.attr_name = attr_name
@@ -101,7 +105,7 @@ class MetaParameter(NrnParameter):
 class NrnMetaListEqualParameter(bluepyopt.parameters.MetaListEqualParameter):
     """Nrn version of MetaListEqualParameter, implements instantiate"""
 
-    def instantiate(self, sim=None, icell=None):
+    def instantiate(self, sim=None, icell=None, params=None):
         """Instantiate"""
 
         for sub_parameter in self.sub_parameters:
@@ -126,7 +130,8 @@ class NrnGlobalParameter(NrnParameter, DictMixin):
             value=None,
             frozen=False,
             bounds=None,
-            param_name=None):
+            param_name=None,
+            param_dependencies=None):
         """Contructor
 
         Args:
@@ -143,11 +148,12 @@ class NrnGlobalParameter(NrnParameter, DictMixin):
             name,
             value=value,
             frozen=frozen,
-            bounds=bounds)
+            bounds=bounds,
+            param_dependencies=param_dependencies)
 
         self.param_name = param_name
 
-    def instantiate(self, sim=None, icell=None):
+    def instantiate(self, sim=None, icell=None, params=None):
         """Instantiate"""
 
         setattr(sim.neuron.h, self.param_name, self.value)
@@ -165,7 +171,7 @@ class NrnSectionParameter(NrnParameter, DictMixin):
 
     """Parameter of a section"""
     SERIALIZED_FIELDS = ('name', 'value', 'frozen', 'bounds', 'param_name',
-                         'value_scaler', 'locations', )
+                         'value_scaler', 'locations', 'param_dependencies')
 
     def __init__(
             self,
@@ -175,7 +181,8 @@ class NrnSectionParameter(NrnParameter, DictMixin):
             bounds=None,
             param_name=None,
             value_scaler=None,
-            locations=None):
+            locations=None,
+            param_dependencies=None):
         """Contructor
 
         Args:
@@ -189,13 +196,16 @@ class NrnSectionParameter(NrnParameter, DictMixin):
             value_scaler (float): value used to scale the parameter value
             locations (list of ephys.locations.Location): locations on which
                 to instantiate the parameter
+            param_dependencies (list): dependencies needed to instantiate
+                the parameter
         """
 
         super(NrnSectionParameter, self).__init__(
             name,
             value=value,
             frozen=frozen,
-            bounds=bounds)
+            bounds=bounds,
+            param_dependencies=param_dependencies)
 
         self.locations = locations
         self.param_name = param_name
@@ -206,7 +216,7 @@ class NrnSectionParameter(NrnParameter, DictMixin):
             self.value_scaler = parameterscalers.NrnSegmentLinearScaler()
         self.value_scale_func = self.value_scaler.scale
 
-    def instantiate(self, sim=None, icell=None):
+    def instantiate(self, sim=None, icell=None, params=None):
         """Instantiate"""
         if self.value is None:
             raise Exception(
@@ -214,11 +224,15 @@ class NrnSectionParameter(NrnParameter, DictMixin):
                 ' without value' %
                 self.name)
 
+        _values = {"value": self.value}
+        for param in self.param_dependencies:
+            _values[param] = params[param].value
+
         for location in self.locations:
             iseclist = location.instantiate(sim=sim, icell=icell)
             for section in iseclist:
                 setattr(section, self.param_name,
-                        self.value_scale_func(self.value, section, sim=sim))
+                        self.value_scale_func(_values, section, sim=sim))
             logger.debug(
                 'Set %s in %s to %s',
                 self.param_name,
@@ -238,7 +252,8 @@ class NrnPointProcessParameter(NrnParameter, DictMixin):
 
     """Parameter of a section"""
     SERIALIZED_FIELDS = ('name', 'value', 'frozen', 'bounds', 'param_name',
-                         'value_scaler', 'locations', )
+                         'value_scaler', 'locations', 'param_name',
+                         'param_dependancies')
 
     def __init__(
             self,
@@ -247,7 +262,8 @@ class NrnPointProcessParameter(NrnParameter, DictMixin):
             frozen=False,
             bounds=None,
             locations=None,
-            param_name=None):
+            param_name=None,
+            param_dependencies=None):
         """Constructor
 
         Args:
@@ -260,18 +276,21 @@ class NrnPointProcessParameter(NrnParameter, DictMixin):
             locations: an iterator of the point process locations you want to
                        set the parameters of
             param_name (str): name of parameter used within the point process
+            param_dependencies (list): dependencies needed to intantiate
+                the parameter
         """
 
         super(NrnPointProcessParameter, self).__init__(
             name,
             value=value,
             frozen=frozen,
-            bounds=bounds)
+            bounds=bounds,
+            param_dependencies=param_dependencies)
 
         self.locations = locations
         self.param_name = param_name
 
-    def instantiate(self, sim=None, icell=None):
+    def instantiate(self, sim=None, icell=None, params=None):
         """Instantiate"""
         if self.value is None:
             raise Exception(
@@ -301,7 +320,7 @@ class NrnRangeParameter(NrnParameter, DictMixin):
 
     """Parameter that has a range over a section"""
     SERIALIZED_FIELDS = ('name', 'value', 'frozen', 'bounds', 'param_name',
-                         'value_scaler', 'locations', )
+                         'value_scaler', 'locations', 'param_dependencies')
 
     def __init__(
             self,
@@ -311,7 +330,8 @@ class NrnRangeParameter(NrnParameter, DictMixin):
             bounds=None,
             param_name=None,
             value_scaler=None,
-            locations=None):
+            locations=None,
+            param_dependencies=None):
         """Contructor
 
         Args:
@@ -325,13 +345,16 @@ class NrnRangeParameter(NrnParameter, DictMixin):
             value_scaler (float): value used to scale the parameter value
             locations (list of ephys.locations.Location): locations on which
                 to instantiate the parameter
+            param_dependencies (list): dependencies needed to intantiate
+                the parameter
         """
 
         super(NrnRangeParameter, self).__init__(
             name,
             value=value,
             frozen=frozen,
-            bounds=bounds)
+            bounds=bounds,
+            param_dependencies=param_dependencies)
 
         self.locations = locations
         self.param_name = param_name
@@ -341,18 +364,22 @@ class NrnRangeParameter(NrnParameter, DictMixin):
             self.value_scaler = parameterscalers.NrnSegmentLinearScaler()
         self.value_scale_func = self.value_scaler.scale
 
-    def instantiate(self, sim=None, icell=None):
+    def instantiate(self, sim=None, icell=None, params=None):
         """Instantiate"""
         if self.value is None:
             raise Exception(
                 'NrnRangeParameter: impossible to instantiate parameter "%s" '
                 'without value' % self.name)
 
+        _values = {"value": self.value}
+        for param in self.param_dependencies:
+            _values[param] = params[param].value
+
         for location in self.locations:
             for isection in location.instantiate(sim=sim, icell=icell):
                 for seg in isection:
                     setattr(seg, '%s' % self.param_name,
-                            self.value_scale_func(self.value, seg, sim=sim))
+                            self.value_scale_func(_values, seg, sim=sim))
         logger.debug(
             'Set %s in %s to %s with scaler %s', self.param_name,
             [str(location)
